@@ -32,10 +32,11 @@ pos = ax1.imshow(Zpos, cmap='Blues', interpolation='none')
 fig.colorbar(pos, ax=ax1)
 
 # repeat everything above for the negative data
+# you can specify location, anchor and shrink the colorbar
 neg = ax2.imshow(Zneg, cmap='Reds_r', interpolation='none')
-fig.colorbar(neg, ax=ax2)
+fig.colorbar(neg, ax=ax2, location='right', anchor=(0, 0.3), shrink=0.7)
 
-# Plot both positive and negative values betwen +/- 1.2
+# Plot both positive and negative values between +/- 1.2
 pos_neg_clipped = ax3.imshow(Z, cmap='RdBu', vmin=-1.2, vmax=1.2,
                              interpolation='none')
 # Add minorticks on the colorbar to make it easy to read the
