@@ -8,7 +8,6 @@ imshow with masked array input and out-of-range colors.
 The second subplot illustrates the use of BoundaryNorm to
 get a filled contour effect.
 """
-from copy import copy
 
 import numpy as np
 import matplotlib.pyplot as plt
@@ -25,11 +24,7 @@ Z2 = np.exp(-(X - 1)**2 - (Y - 1)**2)
 Z = (Z1 - Z2) * 2
 
 # Set up a colormap:
-# use copy so that we do not mutate the global colormap instance
-palette = copy(plt.cm.gray)
-palette.set_over('r', 1.0)
-palette.set_under('g', 1.0)
-palette.set_bad('b', 1.0)
+palette = plt.cm.gray.with_extremes(over='r', under='g', bad='b')
 # Alternatively, we could use
 # palette.set_bad(alpha = 0.0)
 # to make the bad region transparent.  This is the default.
@@ -42,10 +37,10 @@ Zm = np.ma.masked_where(Z > 1.2, Z)
 # range to which the regular palette color scale is applied.
 # Anything above that range is colored based on palette.set_over, etc.
 
-# set up the axes
+# set up the Axes objects
 fig, (ax1, ax2) = plt.subplots(nrows=2, figsize=(6, 5.4))
 
-# plot using 'continuous' color map
+# plot using 'continuous' colormap
 im = ax1.imshow(Zm, interpolation='bilinear',
                 cmap=palette,
                 norm=colors.Normalize(vmin=-1.0, vmax=1.0),
@@ -73,3 +68,21 @@ cbar.set_label('proportional')
 
 fig.suptitle('imshow, with out-of-range and masked data')
 plt.show()
+
+#############################################################################
+#
+# ------------
+#
+# References
+# """"""""""
+#
+# The use of the following functions and methods is shown
+# in this example:
+
+import matplotlib
+matplotlib.axes.Axes.imshow
+matplotlib.pyplot.imshow
+matplotlib.figure.Figure.colorbar
+matplotlib.pyplot.colorbar
+matplotlib.colors.BoundaryNorm
+matplotlib.colorbar.ColorbarBase.set_label
