@@ -1,33 +1,32 @@
-'''
+"""
 ================
 Lorenz Attractor
 ================
 
-This is an example of plotting Edward Lorenz's 1963 `"Deterministic
-Nonperiodic Flow"
-<http://journals.ametsoc.org/doi/abs/10.1175/1520-0469%281963%29020%3C0130%3ADNF%3E2.0.CO%3B2>`_
-in a 3-dimensional space using mplot3d.
+This is an example of plotting Edward Lorenz's 1963 `"Deterministic Nonperiodic
+Flow"`_ in a 3-dimensional space using mplot3d.
 
-Note: Because this is a simple non-linear ODE, it would be more easily
-      done using SciPy's ode solver, but this approach depends only
-      upon NumPy.
-'''
+.. _"Deterministic Nonperiodic Flow":
+   https://journals.ametsoc.org/jas/article/20/2/130/16956/Deterministic-Nonperiodic-Flow
+
+.. note::
+   Because this is a simple non-linear ODE, it would be more easily done using
+   SciPy's ODE solver, but this approach depends only upon NumPy.
+"""
 
 import numpy as np
 import matplotlib.pyplot as plt
-# This import registers the 3D projection, but is otherwise unused.
-from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
 
 
 def lorenz(x, y, z, s=10, r=28, b=2.667):
-    '''
+    """
     Given:
        x, y, z: a point of interest in three dimensional space
        s, r, b: parameters defining the lorenz attractor
     Returns:
        x_dot, y_dot, z_dot: values of the lorenz attractor's partial
            derivatives at the point x, y, z
-    '''
+    """
     x_dot = s*(y - x)
     y_dot = r*x - y - x*z
     z_dot = x*y - b*z
@@ -38,9 +37,9 @@ dt = 0.01
 num_steps = 10000
 
 # Need one more for the initial values
-xs = np.empty((num_steps + 1,))
-ys = np.empty((num_steps + 1,))
-zs = np.empty((num_steps + 1,))
+xs = np.empty(num_steps + 1)
+ys = np.empty(num_steps + 1)
+zs = np.empty(num_steps + 1)
 
 # Set initial values
 xs[0], ys[0], zs[0] = (0., 1., 1.05)
@@ -55,8 +54,7 @@ for i in range(num_steps):
 
 
 # Plot
-fig = plt.figure()
-ax = fig.gca(projection='3d')
+ax = plt.figure().add_subplot(projection='3d')
 
 ax.plot(xs, ys, zs, lw=0.5)
 ax.set_xlabel("X Axis")
