@@ -1,7 +1,9 @@
 """
-===============
-Fancyarrow Demo
-===============
+================================
+Annotation arrow style reference
+================================
+
+Overview of the arrow styles available in `~.Axes.annotate`.
 
 """
 import matplotlib.patches as mpatches
@@ -12,11 +14,11 @@ styles = mpatches.ArrowStyle.get_styles()
 ncol = 2
 nrow = (len(styles) + 1) // ncol
 figheight = (nrow + 0.5)
-fig1 = plt.figure(1, (4 * ncol / 1.5, figheight / 1.5))
+fig = plt.figure(figsize=(4 * ncol / 1.5, figheight / 1.5))
 fontsize = 0.2 * 70
 
 
-ax = fig1.add_axes([0, 0, 1, 1], frameon=False, aspect=1.)
+ax = fig.add_axes([0, 0, 1, 1], frameon=False, aspect=1.)
 
 ax.set_xlim(0, 4 * ncol)
 ax.set_ylim(0, figheight)
@@ -52,3 +54,19 @@ ax.xaxis.set_visible(False)
 ax.yaxis.set_visible(False)
 
 plt.show()
+
+#############################################################################
+#
+# ------------
+#
+# References
+# """"""""""
+#
+# The use of the following functions, methods, classes and modules is shown
+# in this example:
+
+import matplotlib
+matplotlib.patches
+matplotlib.patches.ArrowStyle
+matplotlib.patches.ArrowStyle.get_styles
+matplotlib.axes.Axes.annotate
