@@ -1,19 +1,17 @@
 """
-====================
-Connectionstyle Demo
-====================
+=================================
+Connection styles for annotations
+=================================
 
+When creating an annotation using `~.Axes.annotate`, the arrow shape can be
+controlled via the *connectionstyle* parameter of *arrowprops*. For further
+details see the description of `.FancyArrowPatch`.
 """
 
 import matplotlib.pyplot as plt
 
 
-fig, axs = plt.subplots(3, 5, figsize=(8, 4.8))
-x1, y1 = 0.3, 0.3
-x2, y2 = 0.7, 0.7
-
-
-def demo_con_style(ax, connectionstyle, label=None):
+def demo_con_style(ax, connectionstyle):
     x1, y1 = 0.3, 0.2
     x2, y2 = 0.8, 0.6
 
@@ -21,11 +19,9 @@ def demo_con_style(ax, connectionstyle, label=None):
     ax.annotate("",
                 xy=(x1, y1), xycoords='data',
                 xytext=(x2, y2), textcoords='data',
-                arrowprops=dict(arrowstyle="->",
-                                color="0.5",
+                arrowprops=dict(arrowstyle="->", color="0.5",
                                 shrinkA=5, shrinkB=5,
-                                patchA=None,
-                                patchB=None,
+                                patchA=None, patchB=None,
                                 connectionstyle=connectionstyle,
                                 ),
                 )
@@ -34,6 +30,7 @@ def demo_con_style(ax, connectionstyle, label=None):
             transform=ax.transAxes, ha="left", va="top")
 
 
+fig, axs = plt.subplots(3, 5, figsize=(8, 4.8))
 demo_con_style(axs[0, 0], "angle3,angleA=90,angleB=0")
 demo_con_style(axs[1, 0], "angle3,angleA=0,angleB=90")
 demo_con_style(axs[0, 1], "arc3,rad=0.")
@@ -51,6 +48,20 @@ demo_con_style(axs[2, 4], "bar,angle=180,fraction=-0.2")
 
 for ax in axs.flat:
     ax.set(xlim=(0, 1), ylim=(0, 1), xticks=[], yticks=[], aspect=1)
-fig.tight_layout(pad=0)
+fig.tight_layout(pad=0.2)
 
 plt.show()
+
+#############################################################################
+#
+# ------------
+#
+# References
+# """"""""""
+#
+# The use of the following functions, methods, classes and modules is shown
+# in this example:
+
+import matplotlib
+matplotlib.axes.Axes.annotate
+matplotlib.patches.FancyArrowPatch
