@@ -1,3 +1,13 @@
+"""
+=======
+Buttons
+=======
+
+Constructing a simple button GUI to modify a sine wave.
+
+The ``next`` and ``previous`` button widget helps visualize the wave with
+new frequencies.
+"""
 
 import numpy as np
 import matplotlib.pyplot as plt
@@ -12,7 +22,7 @@ s = np.sin(2*np.pi*freqs[0]*t)
 l, = plt.plot(t, s, lw=2)
 
 
-class Index(object):
+class Index:
     ind = 0
 
     def next(self, event):
@@ -38,3 +48,16 @@ bprev = Button(axprev, 'Previous')
 bprev.on_clicked(callback.prev)
 
 plt.show()
+
+#############################################################################
+#
+# ------------
+#
+# References
+# """"""""""
+#
+# The use of the following functions, methods, classes and modules is shown
+# in this example:
+
+import matplotlib
+matplotlib.widgets.Button
