@@ -1,38 +1,45 @@
 from collections import OrderedDict
 import copy
 import os
+from pathlib import Path
+import subprocess
+import sys
 from unittest import mock
-import warnings
 
 from cycler import cycler, Cycler
 import pytest
 
 import matplotlib as mpl
-from matplotlib.cbook import MatplotlibDeprecationWarning
+from matplotlib import _api, _c_internal_utils
 import matplotlib.pyplot as plt
 import matplotlib.colors as mcolors
 import numpy as np
-from matplotlib.rcsetup import (validate_bool_maybe_none,
-                                validate_stringlist,
-                                validate_colorlist,
-                                validate_color,
-                                validate_bool,
-                                validate_nseq_int,
-                                validate_nseq_float,
-                                validate_cycler,
-                                validate_hatch,
-                                validate_hist_bins,
-                                validate_markevery,
-                                _validate_linestyle)
+from matplotlib.rcsetup import (
+    validate_bool,
+    validate_bool_maybe_none,
+    validate_color,
+    validate_colorlist,
+    validate_cycler,
+    validate_float,
+    validate_fontweight,
+    validate_hatch,
+    validate_hist_bins,
+    validate_int,
+    validate_markevery,
+    validate_stringlist,
+    _validate_linestyle,
+    _listify_validator)
 
 
-def test_rcparams():
+def test_rcparams(tmpdir):
     mpl.rc('text', usetex=False)
     mpl.rc('lines', linewidth=22)
 
     usetex = mpl.rcParams['text.usetex']
     linewidth = mpl.rcParams['lines.linewidth']
-    fname = os.path.join(os.path.dirname(__file__), 'test_rcparams.rc')
+
+    rcpath = Path(tmpdir) / 'test_rcparams.rc'
+    rcpath.write_text('lines.linewidth: 33')
 
     # test context given dictionary
     with mpl.rc_context(rc={'text.usetex': not usetex}):
@@ -40,17 +47,25 @@ def test_rcparams():
     assert mpl.rcParams['text.usetex'] == usetex
 
     # test context given filename (mpl.rc sets linewidth to 33)
-    with mpl.rc_context(fname=fname):
+    with mpl.rc_context(fname=rcpath):
         assert mpl.rcParams['lines.linewidth'] == 33
     assert mpl.rcParams['lines.linewidth'] == linewidth
 
     # test context given filename and dictionary
-    with mpl.rc_context(fname=fname, rc={'lines.linewidth': 44}):
+    with mpl.rc_context(fname=rcpath, rc={'lines.linewidth': 44}):
         assert mpl.rcParams['lines.linewidth'] == 44
     assert mpl.rcParams['lines.linewidth'] == linewidth
 
+    # test context as decorator (and test reusability, by calling func twice)
+    @mpl.rc_context({'lines.linewidth': 44})
+    def func():
+        assert mpl.rcParams['lines.linewidth'] == 44
+
+    func()
+    func()
+
     # test rc_file
-    mpl.rc_file(fname)
+    mpl.rc_file(rcpath)
     assert mpl.rcParams['lines.linewidth'] == 33
 
 
@@ -91,22 +106,15 @@ def test_rcparams_update():
     rc = mpl.RcParams({'figure.figsize': (3.5, 42)})
     bad_dict = {'figure.figsize': (3.5, 42, 1)}
     # make sure validation happens on input
-    with pytest.raises(ValueError):
-
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore',
-                                message='.*(validate)',
-                                category=UserWarning)
-            rc.update(bad_dict)
+    with pytest.raises(ValueError), \
+         pytest.warns(UserWarning, match="validate"):
+        rc.update(bad_dict)
 
 
 def test_rcparams_init():
-    with pytest.raises(ValueError):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore',
-                                message='.*(validate)',
-                                category=UserWarning)
-            mpl.RcParams({'figure.figsize': (3.5, 42, 1)})
+    with pytest.raises(ValueError), \
+         pytest.warns(UserWarning, match="validate"):
+        mpl.RcParams({'figure.figsize': (3.5, 42, 1)})
 
 
 def test_Bug_2543():
@@ -117,21 +125,18 @@ def test_Bug_2543():
     # We filter warnings at this stage since a number of them are raised
     # for deprecated rcparams as they should. We don't want these in the
     # printed in the test suite.
-    with warnings.catch_warnings():
-        warnings.filterwarnings('ignore',
-                                category=MatplotlibDeprecationWarning)
+    with _api.suppress_matplotlib_deprecation_warning():
         with mpl.rc_context():
             _copy = mpl.rcParams.copy()
             for key in _copy:
                 mpl.rcParams[key] = _copy[key]
         with mpl.rc_context():
-            _deep_copy = copy.deepcopy(mpl.rcParams)
+            copy.deepcopy(mpl.rcParams)
         # real test is that this does not raise
         assert validate_bool_maybe_none(None) is None
         assert validate_bool_maybe_none("none") is None
-
-    with pytest.raises(ValueError):
-        validate_bool_maybe_none("blah")
+        with pytest.raises(ValueError):
+            validate_bool_maybe_none("blah")
     with pytest.raises(ValueError):
         validate_bool(None)
     with pytest.raises(ValueError):
@@ -184,14 +189,30 @@ def test_mec_rcparams():
     assert ln.get_markeredgecolor() == 'r'
 
 
-def test_Issue_1713():
-    utf32_be = os.path.join(os.path.dirname(__file__),
-                           'test_utf32_be_rcparams.rc')
+def test_axes_titlecolor_rcparams():
+    mpl.rcParams['axes.titlecolor'] = 'r'
+    _, ax = plt.subplots()
+    title = ax.set_title("Title")
+    assert title.get_color() == 'r'
+
+
+def test_Issue_1713(tmpdir):
+    rcpath = Path(tmpdir) / 'test_rcparams.rc'
+    rcpath.write_text('timezone: UTC', encoding='UTF-32-BE')
     with mock.patch('locale.getpreferredencoding', return_value='UTF-32-BE'):
-        rc = mpl.rc_params_from_file(utf32_be, True, False)
+        rc = mpl.rc_params_from_file(rcpath, True, False)
     assert rc.get('timezone') == 'UTC'
 
 
+def test_animation_frame_formats():
+    # Animation frame_format should allow any of the following
+    # if any of these are not allowed, an exception will be raised
+    # test for gh issue #17908
+    for fmt in ['png', 'jpeg', 'tiff', 'raw', 'rgba', 'ppm',
+                'sgi', 'bmp', 'pbm', 'svg']:
+        mpl.rcParams['animation.frame_format'] = fmt
+
+
 def generate_validator_testcases(valid):
     validation_tests = (
         {'validator': validate_bool,
@@ -200,7 +221,8 @@ def generate_validator_testcases(valid):
                      *((_, False) for _ in
                        ('f', 'n', 'no', 'off', 'false', '0', 0, False))),
          'fail': ((_, ValueError)
-                  for _ in ('aardvark', 2, -1, [], ))},
+                  for _ in ('aardvark', 2, -1, [], ))
+         },
         {'validator': validate_stringlist,
          'success': (('', []),
                      ('a,b', ['a', 'b']),
@@ -213,12 +235,12 @@ def generate_validator_testcases(valid):
                      (np.array(['a', 'b']), ['a', 'b']),
                      ((1, 2), ['1', '2']),
                      (np.array([1, 2]), ['1', '2']),
-                    ),
-         'fail': ((dict(), ValueError),
+                     ),
+         'fail': ((set(), ValueError),
                   (1, ValueError),
-                 )
-        },
-        {'validator': validate_nseq_int(2),
+                  )
+         },
+        {'validator': _listify_validator(validate_int, n=2),
          'success': ((_, [1, 2])
                      for _ in ('1, 2', [1.5, 2.5], [1, 2],
                                (1, 2), np.array((1, 2)))),
@@ -226,16 +248,14 @@ def generate_validator_testcases(valid):
                   for _ in ('aardvark', ('a', 1),
                             (1, 2, 3)
                             ))
-        },
-        {'validator': validate_nseq_float(2),
+         },
+        {'validator': _listify_validator(validate_float, n=2),
          'success': ((_, [1.5, 2.5])
                      for _ in ('1.5, 2.5', [1.5, 2.5], [1.5, 2.5],
                                (1.5, 2.5), np.array((1.5, 2.5)))),
          'fail': ((_, ValueError)
-                  for _ in ('aardvark', ('a', 1),
-                            (1, 2, 3)
-                            ))
-        },
+                  for _ in ('aardvark', ('a', 1), (1, 2, 3), (None, ), None))
+         },
         {'validator': validate_cycler,
          'success': (('cycler("color", "rgb")',
                       cycler("color", 'rgb')),
@@ -244,17 +264,17 @@ def generate_validator_testcases(valid):
                      ("""(cycler("color", ["r", "g", "b"]) +
                           cycler("mew", [2, 3, 5]))""",
                       (cycler("color", 'rgb') +
-                          cycler("markeredgewidth", [2, 3, 5]))),
+                       cycler("markeredgewidth", [2, 3, 5]))),
                      ("cycler(c='rgb', lw=[1, 2, 3])",
                       cycler('color', 'rgb') + cycler('linewidth', [1, 2, 3])),
                      ("cycler('c', 'rgb') * cycler('linestyle', ['-', '--'])",
                       (cycler('color', 'rgb') *
-                          cycler('linestyle', ['-', '--']))),
+                       cycler('linestyle', ['-', '--']))),
                      (cycler('ls', ['-', '--']),
                       cycler('linestyle', ['-', '--'])),
                      (cycler(mew=[2, 5]),
                       cycler('markeredgewidth', [2, 5])),
-                    ),
+                     ),
          # This is *so* incredibly important: validate_cycler() eval's
          # an arbitrary string! I think I have it locked down enough,
          # and that is what this is testing.
@@ -265,40 +285,40 @@ def generate_validator_testcases(valid):
          'fail': ((4, ValueError),  # Gotta be a string or Cycler object
                   ('cycler("bleh, [])', ValueError),  # syntax error
                   ('Cycler("linewidth", [1, 2, 3])',
-                      ValueError),  # only 'cycler()' function is allowed
+                   ValueError),  # only 'cycler()' function is allowed
                   ('1 + 2', ValueError),  # doesn't produce a Cycler object
                   ('os.system("echo Gotcha")', ValueError),  # os not available
                   ('import os', ValueError),  # should not be able to import
                   ('def badjuju(a): return a; badjuju(cycler("color", "rgb"))',
-                      ValueError),  # Should not be able to define anything
-                                    # even if it does return a cycler
+                   ValueError),  # Should not be able to define anything
+                  # even if it does return a cycler
                   ('cycler("waka", [1, 2, 3])', ValueError),  # not a property
                   ('cycler(c=[1, 2, 3])', ValueError),  # invalid values
                   ("cycler(lw=['a', 'b', 'c'])", ValueError),  # invalid values
                   (cycler('waka', [1, 3, 5]), ValueError),  # not a property
                   (cycler('color', ['C1', 'r', 'g']), ValueError)  # no CN
-                 )
-        },
+                  )
+         },
         {'validator': validate_hatch,
          'success': (('--|', '--|'), ('\\oO', '\\oO'),
                      ('/+*/.x', '/+*/.x'), ('', '')),
          'fail': (('--_', ValueError),
                   (8, ValueError),
                   ('X', ValueError)),
-        },
+         },
         {'validator': validate_colorlist,
          'success': (('r,g,b', ['r', 'g', 'b']),
                      (['r', 'g', 'b'], ['r', 'g', 'b']),
                      ('r, ,', ['r']),
                      (['', 'g', 'blue'], ['g', 'blue']),
                      ([np.array([1, 0, 0]), np.array([0, 1, 0])],
-                         np.array([[1, 0, 0], [0, 1, 0]])),
+                     np.array([[1, 0, 0], [0, 1, 0]])),
                      (np.array([[1, 0, 0], [0, 1, 0]]),
-                         np.array([[1, 0, 0], [0, 1, 0]])),
-                    ),
+                     np.array([[1, 0, 0], [0, 1, 0]])),
+                     ),
          'fail': (('fish', ValueError),
-                 ),
-        },
+                  ),
+         },
         {'validator': validate_color,
          'success': (('None', 'none'),
                      ('none', 'none'),
@@ -306,18 +326,18 @@ def generate_validator_testcases(valid):
                      ('AABBCC00', '#AABBCC00'),  # RGBA hex code
                      ('tab:blue', 'tab:blue'),  # named color
                      ('C12', 'C12'),  # color from cycle
-                     ('(0, 1, 0)', [0.0, 1.0, 0.0]),  # RGB tuple
+                     ('(0, 1, 0)', (0.0, 1.0, 0.0)),  # RGB tuple
                      ((0, 1, 0), (0, 1, 0)),  # non-string version
-                     ('(0, 1, 0, 1)', [0.0, 1.0, 0.0, 1.0]),  # RGBA tuple
+                     ('(0, 1, 0, 1)', (0.0, 1.0, 0.0, 1.0)),  # RGBA tuple
                      ((0, 1, 0, 1), (0, 1, 0, 1)),  # non-string version
-                     ('(0, 1, "0.5")', [0.0, 1.0, 0.5]),  # unusual but valid
-                    ),
+                     ),
          'fail': (('tab:veryblue', ValueError),  # invalid name
                   ('(0, 1)', ValueError),  # tuple with length < 3
                   ('(0, 1, 0, 1, 0)', ValueError),  # tuple with length > 4
                   ('(0, 1, none)', ValueError),  # cannot cast none to float
-                 ),
-        },
+                  ('(0, 1, "0.5")', ValueError),  # last one not a float
+                  ),
+         },
         {'validator': validate_hist_bins,
          'success': (('auto', 'auto'),
                      ('fd', 'fd'),
@@ -328,7 +348,7 @@ def generate_validator_testcases(valid):
                      ),
          'fail': (('aardvark', ValueError),
                   )
-        },
+         },
         {'validator': validate_markevery,
          'success': ((None, None),
                      (1, 1),
@@ -357,7 +377,7 @@ def generate_validator_testcases(valid):
                   ('a', TypeError),
                   (object(), TypeError)
                   )
-        },
+         },
         {'validator': _validate_linestyle,
          'success': (('-', '-'), ('solid', 'solid'),
                      ('--', '--'), ('dashed', 'dashed'),
@@ -366,20 +386,22 @@ def generate_validator_testcases(valid):
                      ('', ''), (' ', ' '),
                      ('None', 'none'), ('none', 'none'),
                      ('DoTtEd', 'dotted'),  # case-insensitive
-                     (['1.23', '4.56'], (None, [1.23, 4.56])),
-                     ([1.23, 456], (None, [1.23, 456.0])),
-                     ([1, 2, 3, 4], (None, [1.0, 2.0, 3.0, 4.0])),
+                     ('1, 3', (0, (1, 3))),
+                     ([1.23, 456], (0, [1.23, 456.0])),
+                     ([1, 2, 3, 4], (0, [1.0, 2.0, 3.0, 4.0])),
+                     ((0, [1, 2]), (0, [1, 2])),
+                     ((-1, [1, 2]), (-1, [1, 2])),
                      ),
          'fail': (('aardvark', ValueError),  # not a valid string
                   (b'dotted', ValueError),
                   ('dotted'.encode('utf-16'), ValueError),
-                  ((None, [1, 2]), ValueError),  # (offset, dashes) != OK
-                  ((0, [1, 2]), ValueError),  # idem
-                  ((-1, [1, 2]), ValueError),  # idem
                   ([1, 2, 3], ValueError),  # sequence with odd length
                   (1.23, ValueError),  # not a sequence
+                  (("a", [1, 2]), ValueError),  # wrong explicit offset
+                  ((1, [1, 2, 3]), ValueError),  # odd length sequence
+                  (([1, 2], 1), ValueError),  # inverted offset/onoff
                   )
-        },
+         },
     )
 
     for validator_dict in validation_tests:
@@ -397,7 +419,7 @@ def generate_validator_testcases(valid):
 def test_validator_valid(validator, arg, target):
     res = validator(arg)
     if isinstance(target, np.ndarray):
-        assert np.all(res == target)
+        np.testing.assert_equal(res, target)
     elif not isinstance(target, Cycler):
         assert res == target
     else:
@@ -412,6 +434,26 @@ def test_validator_invalid(validator, arg, exception_type):
         validator(arg)
 
 
+@pytest.mark.parametrize('weight, parsed_weight', [
+    ('bold', 'bold'),
+    ('BOLD', ValueError),  # weight is case-sensitive
+    (100, 100),
+    ('100', 100),
+    (np.array(100), 100),
+    # fractional fontweights are not defined. This should actually raise a
+    # ValueError, but historically did not.
+    (20.6, 20),
+    ('20.6', ValueError),
+    ([100], ValueError),
+])
+def test_validate_fontweight(weight, parsed_weight):
+    if parsed_weight is ValueError:
+        with pytest.raises(ValueError):
+            validate_fontweight(weight)
+    else:
+        assert validate_fontweight(weight) == parsed_weight
+
+
 def test_keymaps():
     key_list = [k for k in mpl.rcParams if 'keymap' in k]
     for k in key_list:
@@ -419,73 +461,43 @@ def test_keymaps():
 
 
 def test_rcparams_reset_after_fail():
-
     # There was previously a bug that meant that if rc_context failed and
     # raised an exception due to issues in the supplied rc parameters, the
     # global rc parameters were left in a modified state.
-
     with mpl.rc_context(rc={'text.usetex': False}):
-
         assert mpl.rcParams['text.usetex'] is False
-
         with pytest.raises(KeyError):
             with mpl.rc_context(rc=OrderedDict([('text.usetex', True),
                                                 ('test.blah', True)])):
                 pass
-
         assert mpl.rcParams['text.usetex'] is False
 
 
-def test_if_rctemplate_is_up_to_date():
-    # This tests if the matplotlibrc.template file contains all valid rcParams.
-    deprecated = {*mpl._all_deprecated, *mpl._deprecated_remain_as_none}
-    path_to_rc = os.path.join(mpl.get_data_path(), 'matplotlibrc')
-    with open(path_to_rc, "r") as f:
-        rclines = f.readlines()
-    missing = {}
-    for k, v in mpl.defaultParams.items():
-        if k[0] == "_":
-            continue
-        if k in deprecated:
-            continue
-        if k.startswith(
-                ("verbose.", "examples.directory", "text.latex.unicode")):
-            continue
-        found = False
-        for line in rclines:
-            if k in line:
-                found = True
-        if not found:
-            missing.update({k: v})
-    if missing:
-        raise ValueError("The following params are missing in the "
-                         "matplotlibrc.template file: {}"
-                         .format(missing.items()))
+@pytest.mark.skipif(sys.platform != "linux", reason="Linux only")
+def test_backend_fallback_headless(tmpdir):
+    env = {**os.environ,
+           "DISPLAY": "", "WAYLAND_DISPLAY": "",
+           "MPLBACKEND": "", "MPLCONFIGDIR": str(tmpdir)}
+    with pytest.raises(subprocess.CalledProcessError):
+        subprocess.run(
+            [sys.executable, "-c",
+             ("import matplotlib;" +
+              "matplotlib.use('tkagg');" +
+              "import matplotlib.pyplot")
+             ],
+            env=env, check=True)
 
 
-def test_if_rctemplate_would_be_valid(tmpdir):
-    # This tests if the matplotlibrc.template file would result in a valid
-    # rc file if all lines are uncommented.
-    path_to_rc = os.path.join(mpl.get_data_path(), 'matplotlibrc')
-    with open(path_to_rc, "r") as f:
-        rclines = f.readlines()
-    newlines = []
-    for line in rclines:
-        if line[0] == "#":
-            newline = line[1:]
-        else:
-            newline = line
-        if "$TEMPLATE_BACKEND" in newline:
-            newline = "backend : Agg"
-        if "datapath" in newline:
-            newline = ""
-        newlines.append(newline)
-    d = tmpdir.mkdir('test1')
-    fname = str(d.join('testrcvalid.temp'))
-    with open(fname, "w") as f:
-        f.writelines(newlines)
-    with pytest.warns(None) as record:
-        mpl.rc_params_from_file(fname,
-                                fail_on_error=True,
-                                use_default_template=False)
-        assert len(record) == 0
+@pytest.mark.skipif(
+    sys.platform == "linux" and not _c_internal_utils.display_is_valid(),
+    reason="headless")
+def test_backend_fallback_headful(tmpdir):
+    pytest.importorskip("tkinter")
+    env = {**os.environ, "MPLBACKEND": "", "MPLCONFIGDIR": str(tmpdir)}
+    backend = subprocess.check_output(
+        [sys.executable, "-c",
+         "import matplotlib.pyplot; print(matplotlib.get_backend())"],
+        env=env, universal_newlines=True)
+    # The actual backend will depend on what's installed, but at least tkagg is
+    # present.
+    assert backend.strip().lower() != "agg"
