@@ -1,83 +1,14 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-try:
-    # mock in python 3.3+
-    from unittest import mock
-except ImportError:
-    import mock
-
+import matplotlib.colors as mcolors
 import matplotlib.widgets as widgets
 import matplotlib.pyplot as plt
 from matplotlib.testing.decorators import image_comparison
+from matplotlib.testing.widgets import do_event, get_ax
 
 from numpy.testing import assert_allclose
 
 import pytest
 
 
-def get_ax():
-    fig, ax = plt.subplots(1, 1)
-    ax.plot([0, 200], [0, 200])
-    ax.set_aspect(1.0)
-    ax.figure.canvas.draw()
-    return ax
-
-
-def do_event(tool, etype, button=1, xdata=0, ydata=0, key=None, step=1):
-    """
-     *name*
-        the event name
-
-    *canvas*
-        the FigureCanvas instance generating the event
-
-    *guiEvent*
-        the GUI event that triggered the matplotlib event
-
-    *x*
-        x position - pixels from left of canvas
-
-    *y*
-        y position - pixels from bottom of canvas
-
-    *inaxes*
-        the :class:`~matplotlib.axes.Axes` instance if mouse is over axes
-
-    *xdata*
-        x coord of mouse in data coords
-
-    *ydata*
-        y coord of mouse in data coords
-
-     *button*
-        button pressed None, 1, 2, 3, 'up', 'down' (up and down are used
-        for scroll events)
-
-    *key*
-        the key depressed when the mouse event triggered (see
-        :class:`KeyEvent`)
-
-    *step*
-        number of scroll steps (positive for 'up', negative for 'down')
-    """
-    event = mock.Mock()
-    event.button = button
-    ax = tool.ax
-    event.x, event.y = ax.transData.transform([(xdata, ydata),
-                                               (xdata, ydata)])[00]
-    event.xdata, event.ydata = xdata, ydata
-    event.inaxes = ax
-    event.canvas = ax.figure.canvas
-    event.key = key
-    event.step = step
-    event.guiEvent = None
-    event.name = 'Custom'
-
-    func = getattr(tool, etype)
-    func(event)
-
-
 def check_rectangle(**kwargs):
     ax = get_ax()
 
@@ -97,8 +28,9 @@ def check_rectangle(**kwargs):
 
     if kwargs.get('drawtype', None) not in ['line', 'none']:
         assert_allclose(tool.geometry,
-            [[100., 100, 199, 199, 100], [100, 199, 199, 100, 100]],
-            err_msg=tool.geometry)
+                        [[100., 100, 199, 199, 100],
+                         [100, 199, 199, 100, 100]],
+                        err_msg=tool.geometry)
 
     assert ax._got_onselect
 
@@ -125,42 +57,43 @@ def test_ellipse():
 
     # drag the rectangle
     do_event(tool, 'press', xdata=10, ydata=10, button=1,
-                    key=' ')
+             key=' ')
+
     do_event(tool, 'onmove', xdata=30, ydata=30, button=1)
     do_event(tool, 'release', xdata=30, ydata=30, button=1)
-    assert tool.extents == (120, 170, 120, 170), tool.extents
+    assert tool.extents == (120, 170, 120, 170)
 
     # create from center
     do_event(tool, 'on_key_press', xdata=100, ydata=100, button=1,
-                    key='control')
+             key='control')
     do_event(tool, 'press', xdata=100, ydata=100, button=1)
     do_event(tool, 'onmove', xdata=125, ydata=125, button=1)
     do_event(tool, 'release', xdata=125, ydata=125, button=1)
     do_event(tool, 'on_key_release', xdata=100, ydata=100, button=1,
-                    key='control')
-    assert tool.extents == (75, 125, 75, 125), tool.extents
+             key='control')
+    assert tool.extents == (75, 125, 75, 125)
 
     # create a square
     do_event(tool, 'on_key_press', xdata=10, ydata=10, button=1,
-                    key='shift')
+             key='shift')
     do_event(tool, 'press', xdata=10, ydata=10, button=1)
     do_event(tool, 'onmove', xdata=35, ydata=30, button=1)
     do_event(tool, 'release', xdata=35, ydata=30, button=1)
     do_event(tool, 'on_key_release', xdata=10, ydata=10, button=1,
-                    key='shift')
+             key='shift')
     extents = [int(e) for e in tool.extents]
     assert extents == [10, 35, 10, 34]
 
     # create a square from center
     do_event(tool, 'on_key_press', xdata=100, ydata=100, button=1,
-                      key='ctrl+shift')
+             key='ctrl+shift')
     do_event(tool, 'press', xdata=100, ydata=100, button=1)
     do_event(tool, 'onmove', xdata=125, ydata=130, button=1)
     do_event(tool, 'release', xdata=125, ydata=130, button=1)
     do_event(tool, 'on_key_release', xdata=100, ydata=100, button=1,
-                      key='ctrl+shift')
+             key='ctrl+shift')
     extents = [int(e) for e in tool.extents]
-    assert extents == [70, 129, 70, 130], extents
+    assert extents == [70, 129, 70, 130]
 
     assert tool.geometry.shape == (2, 73)
     assert_allclose(tool.geometry[:, 0], [70., 100])
@@ -173,7 +106,9 @@ def test_rectangle_handles():
         pass
 
     tool = widgets.RectangleSelector(ax, onselect=onselect,
-                                     maxdist=10, interactive=True)
+                                     maxdist=10, interactive=True,
+                                     marker_props={'markerfacecolor': 'r',
+                                                   'markeredgecolor': 'b'})
     tool.extents = (100, 150, 100, 150)
 
     assert tool.corners == (
@@ -201,6 +136,12 @@ def test_rectangle_handles():
     do_event(tool, 'release', xdata=100, ydata=100)
     assert tool.extents == (10, 100, 10, 100)
 
+    # Check that marker_props worked.
+    assert mcolors.same_color(
+        tool._corner_handles.artist.get_markerfacecolor(), 'r')
+    assert mcolors.same_color(
+        tool._corner_handles.artist.get_markeredgecolor(), 'b')
+
 
 def check_span(*args, **kwargs):
     ax = get_ax()
@@ -267,9 +208,11 @@ def test_CheckButtons():
     check.disconnect(cid)
 
 
-@image_comparison(baseline_images=['check_radio_buttons'], extensions=['png'],
-                  style='default')
+@image_comparison(['check_radio_buttons.png'], style='mpl20', remove_text=True)
 def test_check_radio_buttons_image():
+    # Remove this line when this test image is regenerated.
+    plt.rcParams['text.kerning_factor'] = 6
+
     get_ax()
     plt.subplots_adjust(left=0.3)
     rax1 = plt.axes([0.05, 0.7, 0.15, 0.15])
@@ -279,6 +222,15 @@ def test_check_radio_buttons_image():
                          (False, True, True))
 
 
+@image_comparison(['check_bunch_of_radio_buttons.png'],
+                  style='mpl20', remove_text=True)
+def test_check_bunch_of_radio_buttons():
+    rax = plt.axes([0.05, 0.1, 0.15, 0.7])
+    widgets.RadioButtons(rax, ('B1', 'B2', 'B3', 'B4', 'B5', 'B6',
+                               'B7', 'B8', 'B9', 'B10', 'B11', 'B12',
+                               'B13', 'B14', 'B15'))
+
+
 def test_slider_slidermin_slidermax_invalid():
     fig, ax = plt.subplots()
     # test min/max with floats
@@ -315,8 +267,67 @@ def test_slider_valmin_valmax():
     assert slider.val == slider.valmax
 
 
+def test_slider_valstep_snapping():
+    fig, ax = plt.subplots()
+    slider = widgets.Slider(ax=ax, label='', valmin=0.0, valmax=24.0,
+                            valinit=11.4, valstep=1)
+    assert slider.val == 11
+
+    slider = widgets.Slider(ax=ax, label='', valmin=0.0, valmax=24.0,
+                            valinit=11.4, valstep=[0, 1, 5.5, 19.7])
+    assert slider.val == 5.5
+
+
+def test_slider_horizontal_vertical():
+    fig, ax = plt.subplots()
+    slider = widgets.Slider(ax=ax, label='', valmin=0, valmax=24,
+                            valinit=12, orientation='horizontal')
+    slider.set_val(10)
+    assert slider.val == 10
+    # check the dimension of the slider patch in axes units
+    box = slider.poly.get_extents().transformed(ax.transAxes.inverted())
+    assert_allclose(box.bounds, [0, 0, 10/24, 1])
+
+    fig, ax = plt.subplots()
+    slider = widgets.Slider(ax=ax, label='', valmin=0, valmax=24,
+                            valinit=12, orientation='vertical')
+    slider.set_val(10)
+    assert slider.val == 10
+    # check the dimension of the slider patch in axes units
+    box = slider.poly.get_extents().transformed(ax.transAxes.inverted())
+    assert_allclose(box.bounds, [0, 0, 1, 10/24])
+
+
+@pytest.mark.parametrize("orientation", ["horizontal", "vertical"])
+def test_range_slider(orientation):
+    if orientation == "vertical":
+        idx = [1, 0, 3, 2]
+    else:
+        idx = [0, 1, 2, 3]
+
+    fig, ax = plt.subplots()
+
+    slider = widgets.RangeSlider(
+        ax=ax, label="", valmin=0.0, valmax=1.0, orientation=orientation
+    )
+    box = slider.poly.get_extents().transformed(ax.transAxes.inverted())
+    assert_allclose(box.get_points().flatten()[idx], [0.25, 0, 0.75, 1])
+
+    slider.set_val((0.2, 0.6))
+    assert_allclose(slider.val, (0.2, 0.6))
+    box = slider.poly.get_extents().transformed(ax.transAxes.inverted())
+    assert_allclose(box.get_points().flatten()[idx], [0.2, 0, 0.6, 1])
+
+    slider.set_val((0.2, 0.1))
+    assert_allclose(slider.val, (0.1, 0.2))
+
+    slider.set_val((-1, 10))
+    assert_allclose(slider.val, (0, 1))
+
+
 def check_polygon_selector(event_sequence, expected_result, selections_count):
-    """Helper function to test Polygon Selector
+    """
+    Helper function to test Polygon Selector.
 
     Parameters
     ----------
