@@ -5,13 +5,12 @@ Artist tutorial
 
 Using Artist objects to render on the canvas.
 
-There are three layers to the matplotlib API.
+There are three layers to the Matplotlib API.
 
 * the :class:`matplotlib.backend_bases.FigureCanvas` is the area onto which
   the figure is drawn
-* the :class:`matplotlib.backend_bases.Renderer` is
-  the object which knows how to draw on the
-  :class:`~matplotlib.backend_bases.FigureCanvas`
+* the :class:`matplotlib.backend_bases.Renderer` is the object which knows how
+  to draw on the :class:`~matplotlib.backend_bases.FigureCanvas`
 * and the :class:`matplotlib.artist.Artist` is the object that knows how to use
   a renderer to paint onto the canvas.
 
@@ -47,7 +46,7 @@ for us::
     ax = fig.add_subplot(2, 1, 1) # two rows, one column, first plot
 
 The :class:`~matplotlib.axes.Axes` is probably the most important
-class in the matplotlib API, and the one you will be working with most
+class in the Matplotlib API, and the one you will be working with most
 of the time.  This is because the ``Axes`` is the plotting area into
 which most of the objects go, and the ``Axes`` has many special helper
 methods (:meth:`~matplotlib.axes.Axes.plot`,
@@ -83,18 +82,18 @@ In this example, ``ax`` is the ``Axes`` instance created by the
 ``fig.add_subplot`` call above (remember ``Subplot`` is just a
 subclass of ``Axes``) and when you call ``ax.plot``, it creates a
 ``Line2D`` instance and adds it to the :attr:`Axes.lines
-<matplotlib.axes.Axes.lines>` list.  In the interactive `ipython
-<http://ipython.org/>`_ session below, you can see that the
+<matplotlib.axes.Axes.lines>` list.  In the interactive `IPython
+<https://ipython.org/>`_ session below, you can see that the
 ``Axes.lines`` list is length one and contains the same line that was
 returned by the ``line, = ax.plot...`` call:
 
 .. sourcecode:: ipython
 
     In [101]: ax.lines[0]
-    Out[101]: <matplotlib.lines.Line2D instance at 0x19a95710>
+    Out[101]: <matplotlib.lines.Line2D at 0x19a95710>
 
     In [102]: line
-    Out[102]: <matplotlib.lines.Line2D instance at 0x19a95710>
+    Out[102]: <matplotlib.lines.Line2D at 0x19a95710>
 
 If you make subsequent calls to ``ax.plot`` (and the hold state is "on"
 which is the default) then additional lines will be added to the list.
@@ -107,7 +106,7 @@ of these will work::
 The Axes also has helper methods to configure and decorate the x-axis
 and y-axis tick, tick labels and axis labels::
 
-    xtext = ax.set_xlabel('my xdata') # returns a Text instance
+    xtext = ax.set_xlabel('my xdata')  # returns a Text instance
     ytext = ax.set_ylabel('my ydata')
 
 When you call :meth:`ax.set_xlabel <matplotlib.axes.Axes.set_xlabel>`,
@@ -149,20 +148,20 @@ plt.show()
 # Customizing your objects
 # ========================
 #
-# Every element in the figure is represented by a matplotlib
+# Every element in the figure is represented by a Matplotlib
 # :class:`~matplotlib.artist.Artist`, and each has an extensive list of
 # properties to configure its appearance.  The figure itself contains a
 # :class:`~matplotlib.patches.Rectangle` exactly the size of the figure,
 # which you can use to set the background color and transparency of the
 # figures.  Likewise, each :class:`~matplotlib.axes.Axes` bounding box
-# (the standard white box with black edges in the typical matplotlib
+# (the standard white box with black edges in the typical Matplotlib
 # plot, has a ``Rectangle`` instance that determines the color,
 # transparency, and other properties of the Axes.  These instances are
 # stored as member variables :attr:`Figure.patch
 # <matplotlib.figure.Figure.patch>` and :attr:`Axes.patch
 # <matplotlib.axes.Axes.patch>` ("Patch" is a name inherited from
 # MATLAB, and is a 2D "patch" of color on the figure, e.g., rectangles,
-# circles and polygons).  Every matplotlib ``Artist`` has the following
+# circles and polygons).  Every Matplotlib ``Artist`` has the following
 # properties
 #
 # ==========  =================================================================
@@ -170,7 +169,7 @@ plt.show()
 # ==========  =================================================================
 # alpha       The transparency - a scalar from 0-1
 # animated    A boolean that is used to facilitate animated drawing
-# axes        The axes that the Artist lives in, possibly None
+# axes        The Axes that the Artist lives in, possibly None
 # clip_box    The bounding box that clips the Artist
 # clip_on     Whether clipping is enabled
 # clip_path   The path the artist is clipped to
@@ -183,7 +182,7 @@ plt.show()
 # visible     A boolean whether the artist should be drawn
 # zorder      A number which determines the drawing order
 # rasterized  Boolean; Turns vectors into raster graphics (for compression &
-#             eps transparency)
+#             EPS transparency)
 # ==========  =================================================================
 #
 # Each of the properties is accessed with an old-fashioned setter or
@@ -210,31 +209,49 @@ plt.show()
 # .. sourcecode:: ipython
 #
 #     In [149]: matplotlib.artist.getp(fig.patch)
-# 	alpha = 1.0
-# 	animated = False
-# 	antialiased or aa = True
-# 	axes = None
-# 	clip_box = None
-# 	clip_on = False
-# 	clip_path = None
-# 	contains = None
-# 	edgecolor or ec = w
-# 	facecolor or fc = 0.75
-# 	figure = Figure(8.125x6.125)
-# 	fill = 1
-# 	hatch = None
-# 	height = 1
-# 	label =
-# 	linewidth or lw = 1.0
-# 	picker = None
-# 	transform = <Affine object at 0x134cca84>
-# 	verts = ((0, 0), (0, 1), (1, 1), (1, 0))
-# 	visible = True
-# 	width = 1
-# 	window_extent = <Bbox object at 0x134acbcc>
-# 	x = 0
-# 	y = 0
-# 	zorder = 1
+#       agg_filter = None
+#       alpha = None
+#       animated = False
+#       antialiased or aa = False
+#       bbox = Bbox(x0=0.0, y0=0.0, x1=1.0, y1=1.0)
+#       capstyle = butt
+#       children = []
+#       clip_box = None
+#       clip_on = True
+#       clip_path = None
+#       contains = None
+#       data_transform = BboxTransformTo(     TransformedBbox(         Bbox...
+#       edgecolor or ec = (1.0, 1.0, 1.0, 1.0)
+#       extents = Bbox(x0=0.0, y0=0.0, x1=640.0, y1=480.0)
+#       facecolor or fc = (1.0, 1.0, 1.0, 1.0)
+#       figure = Figure(640x480)
+#       fill = True
+#       gid = None
+#       hatch = None
+#       height = 1
+#       in_layout = False
+#       joinstyle = miter
+#       label =
+#       linestyle or ls = solid
+#       linewidth or lw = 0.0
+#       patch_transform = CompositeGenericTransform(     BboxTransformTo(   ...
+#       path = Path(array([[0., 0.],        [1., 0.],        [1.,...
+#       path_effects = []
+#       picker = None
+#       rasterized = None
+#       sketch_params = None
+#       snap = None
+#       transform = CompositeGenericTransform(     CompositeGenericTra...
+#       transformed_clip_path_and_affine = (None, None)
+#       url = None
+#       verts = [[  0.   0.]  [640.   0.]  [640. 480.]  [  0. 480....
+#       visible = True
+#       width = 1
+#       window_extent = Bbox(x0=0.0, y0=0.0, x1=640.0, y1=480.0)
+#       x = 0
+#       xy = (0, 0)
+#       y = 0
+#       zorder = 1
 #
 # The docstrings for all of the classes also contain the ``Artist``
 # properties, so you can consult the interactive "help" or the
@@ -284,39 +301,38 @@ plt.show()
 #     In [158]: ax2 = fig.add_axes([0.1, 0.1, 0.7, 0.3])
 #
 #     In [159]: ax1
-#     Out[159]: <matplotlib.axes.Subplot instance at 0xd54b26c>
+#     Out[159]: <AxesSubplot:>
 #
 #     In [160]: print(fig.axes)
-#     [<matplotlib.axes.Subplot instance at 0xd54b26c>,
-#     <matplotlib.axes.Axes instance at 0xd3f0b2c>]
+#     [<AxesSubplot:>, <matplotlib.axes._axes.Axes object at 0x7f0768702be0>]
 #
-# Because the figure maintains the concept of the "current axes" (see
+# Because the figure maintains the concept of the "current Axes" (see
 # :meth:`Figure.gca <matplotlib.figure.Figure.gca>` and
 # :meth:`Figure.sca <matplotlib.figure.Figure.sca>`) to support the
-# pylab/pyplot state machine, you should not insert or remove axes
-# directly from the axes list, but rather use the
+# pylab/pyplot state machine, you should not insert or remove Axes
+# directly from the Axes list, but rather use the
 # :meth:`~matplotlib.figure.Figure.add_subplot` and
 # :meth:`~matplotlib.figure.Figure.add_axes` methods to insert, and the
 # :meth:`~matplotlib.figure.Figure.delaxes` method to delete.  You are
-# free however, to iterate over the list of axes or index into it to get
+# free however, to iterate over the list of Axes or index into it to get
 # access to ``Axes`` instances you want to customize.  Here is an
-# example which turns all the axes grids on::
+# example which turns all the Axes grids on::
 #
 #     for ax in fig.axes:
 #         ax.grid(True)
 #
 #
-# The figure also has its own text, lines, patches and images, which you
-# can use to add primitives directly.  The default coordinate system for
-# the ``Figure`` will simply be in pixels (which is not usually what you
-# want) but you can control this by setting the transform property of
-# the ``Artist`` you are adding to the figure.
+# The figure also has its own ``images``, ``lines``, ``patches`` and ``text``
+# attributes, which you can use to add primitives directly. When doing so, the
+# default coordinate system for the ``Figure`` will simply be in pixels (which
+# is not usually what you want). If you instead use Figure-level methods to add
+# Artists (e.g., using `.Figure.text` to add text), then the default coordinate
+# system will be "figure coordinates" where (0, 0) is the bottom-left of the
+# figure and (1, 1) is the top-right of the figure.
 #
-# .. TODO: Is that still true?
-#
-# More useful is "figure coordinates" where (0, 0) is the bottom-left of
-# the figure and (1, 1) is the top-right of the figure which you can
-# obtain by setting the ``Artist`` transform to :attr:`fig.transFigure
+# As with all ``Artist``\s, you can control this coordinate system by setting
+# the transform property. You can explicitly use "figure coordinates" by
+# setting the ``Artist`` transform to :attr:`fig.transFigure
 # <matplotlib.figure.Figure.transFigure>`:
 
 import matplotlib.lines as lines
@@ -330,23 +346,22 @@ fig.lines.extend([l1, l2])
 plt.show()
 
 ###############################################################################
-# Here is a summary of the Artists the figure contains
-#
-# .. TODO: Add xrefs to this table
+# Here is a summary of the Artists the Figure contains
 #
 # ================ ============================================================
 # Figure attribute Description
 # ================ ============================================================
-# axes             A list of Axes instances (includes Subplot)
-# patch            The Rectangle background
-# images           A list of FigureImage patches -
+# axes             A list of `~.axes.Axes` instances (includes Subplot)
+# patch            The `.Rectangle` background
+# images           A list of `.FigureImage` patches -
 #                  useful for raw pixel display
-# legends          A list of Figure Legend instances
-#                  (different from Axes.legends)
-# lines            A list of Figure Line2D instances
-#                  (rarely used, see Axes.lines)
-# patches          A list of Figure patches (rarely used, see Axes.patches)
-# texts            A list Figure Text instances
+# legends          A list of Figure `.Legend` instances
+#                  (different from ``Axes.legends``)
+# lines            A list of Figure `.Line2D` instances
+#                  (rarely used, see ``Axes.lines``)
+# patches          A list of Figure `.Patch`\s
+#                  (rarely used, see ``Axes.patches``)
+# texts            A list Figure `.Text` instances
 # ================ ============================================================
 #
 # .. _axes-container:
@@ -354,7 +369,7 @@ plt.show()
 # Axes container
 # --------------
 #
-# The :class:`matplotlib.axes.Axes` is the center of the matplotlib
+# The :class:`matplotlib.axes.Axes` is the center of the Matplotlib
 # universe -- it contains the vast majority of all the ``Artists`` used
 # in a figure with many helper methods to create and add these
 # ``Artists`` to itself, as well as helper methods to access and
@@ -366,7 +381,7 @@ plt.show()
 # :class:`~matplotlib.patches.Circle` for polar coordinates; this patch
 # determines the shape, background and border of the plotting region::
 #
-#     ax = fig.add_subplot(111)
+#     ax = fig.add_subplot()
 #     rect = ax.patch  # a Rectangle instance
 #     rect.set_facecolor('green')
 #
@@ -391,7 +406,7 @@ plt.show()
 # .. sourcecode:: ipython
 #
 #     In [229]: print(ax.lines)
-#     [<matplotlib.lines.Line2D instance at 0xd378b0c>]
+#     [<matplotlib.lines.Line2D at 0xd378b0c>]
 #
 # Similarly, methods that create patches, like
 # :meth:`~matplotlib.axes.Axes.bar` creates a list of rectangles, will
@@ -403,9 +418,10 @@ plt.show()
 #     In [233]: n, bins, rectangles = ax.hist(np.random.randn(1000), 50)
 #
 #     In [234]: rectangles
-#     Out[234]: <a list of 50 Patch objects>
+#     Out[234]: <BarContainer object of 50 artists>
 #
 #     In [235]: print(len(ax.patches))
+#     Out[235]: 50
 #
 # You should not add objects directly to the ``Axes.lines`` or
 # ``Axes.patches`` lists unless you know exactly what you are doing,
@@ -433,8 +449,8 @@ plt.show()
 #     None
 #
 #     # and the transformation instance is set to the "identity transform"
-#     In [265]: print(rect.get_transform())
-#     <Affine object at 0x13695544>
+#     In [265]: print(rect.get_data_transform())
+#     IdentityTransform()
 #
 #     # now we add the Rectangle to the Axes
 #     In [266]: ax.add_patch(rect)
@@ -445,12 +461,56 @@ plt.show()
 #     Axes(0.125,0.1;0.775x0.8)
 #
 #     # and the transformation has been set too
-#     In [268]: print(rect.get_transform())
-#     <Affine object at 0x15009ca4>
+#     In [268]: print(rect.get_data_transform())
+#     CompositeGenericTransform(
+#         TransformWrapper(
+#             BlendedAffine2D(
+#                 IdentityTransform(),
+#                 IdentityTransform())),
+#         CompositeGenericTransform(
+#             BboxTransformFrom(
+#                 TransformedBbox(
+#                     Bbox(x0=0.0, y0=0.0, x1=1.0, y1=1.0),
+#                     TransformWrapper(
+#                         BlendedAffine2D(
+#                             IdentityTransform(),
+#                             IdentityTransform())))),
+#             BboxTransformTo(
+#                 TransformedBbox(
+#                     Bbox(x0=0.125, y0=0.10999999999999999, x1=0.9, y1=0.88),
+#                     BboxTransformTo(
+#                         TransformedBbox(
+#                             Bbox(x0=0.0, y0=0.0, x1=6.4, y1=4.8),
+#                             Affine2D(
+#                                 [[100.   0.   0.]
+#                                  [  0. 100.   0.]
+#                                  [  0.   0.   1.]])))))))
 #
 #     # the default axes transformation is ax.transData
 #     In [269]: print(ax.transData)
-#     <Affine object at 0x15009ca4>
+#     CompositeGenericTransform(
+#         TransformWrapper(
+#             BlendedAffine2D(
+#                 IdentityTransform(),
+#                 IdentityTransform())),
+#         CompositeGenericTransform(
+#             BboxTransformFrom(
+#                 TransformedBbox(
+#                     Bbox(x0=0.0, y0=0.0, x1=1.0, y1=1.0),
+#                     TransformWrapper(
+#                         BlendedAffine2D(
+#                             IdentityTransform(),
+#                             IdentityTransform())))),
+#             BboxTransformTo(
+#                 TransformedBbox(
+#                     Bbox(x0=0.125, y0=0.10999999999999999, x1=0.9, y1=0.88),
+#                     BboxTransformTo(
+#                         TransformedBbox(
+#                             Bbox(x0=0.0, y0=0.0, x1=6.4, y1=4.8),
+#                             Affine2D(
+#                                 [[100.   0.   0.]
+#                                  [  0. 100.   0.]
+#                                  [  0.   0.   1.]])))))))
 #
 #     # notice that the xlimits of the Axes have not been changed
 #     In [270]: print(ax.get_xlim())
@@ -463,12 +523,12 @@ plt.show()
 #     # we can manually invoke the auto-scaling machinery
 #     In [272]: ax.autoscale_view()
 #
-#     # and now the xlim are updated to encompass the rectangle
+#     # and now the xlim are updated to encompass the rectangle, plus margins
 #     In [273]: print(ax.get_xlim())
-#     (1.0, 6.0)
+#     (0.75, 6.25)
 #
 #     # we have to manually force a figure draw
-#     In [274]: ax.figure.canvas.draw()
+#     In [274]: fig.canvas.draw()
 #
 #
 # There are many, many ``Axes`` helper methods for creating primitive
@@ -476,20 +536,21 @@ plt.show()
 # below summarizes a small sampling of them, the kinds of ``Artist`` they
 # create, and where they store them
 #
-# ==============================  ====================  =======================
-# Helper method                   Artist                Container
-# ==============================  ====================  =======================
-# ax.annotate - text annotations  Annotate              ax.texts
-# ax.bar  - bar charts            Rectangle             ax.patches
-# ax.errorbar - error bar plots   Line2D and Rectangle  ax.lines and ax.patches
-# ax.fill - shared area           Polygon               ax.patches
-# ax.hist - histograms            Rectangle             ax.patches
-# ax.imshow - image data          AxesImage             ax.images
-# ax.legend - axes legends        Legend                ax.legends
-# ax.plot - xy plots              Line2D                ax.lines
-# ax.scatter - scatter charts     PolygonCollection     ax.collections
-# ax.text - text                  Text                  ax.texts
-# ==============================  ====================  =======================
+# =========================================  =================  ===============
+# Axes helper method                         Artist             Container
+# =========================================  =================  ===============
+# `~.axes.Axes.annotate` - text annotations  `.Annotation`      ax.texts
+# `~.axes.Axes.bar` - bar charts             `.Rectangle`       ax.patches
+# `~.axes.Axes.errorbar` - error bar plots   `.Line2D` and      ax.lines and
+#                                            `.Rectangle`       ax.patches
+# `~.axes.Axes.fill` - shared area           `.Polygon`         ax.patches
+# `~.axes.Axes.hist` - histograms            `.Rectangle`       ax.patches
+# `~.axes.Axes.imshow` - image data          `.AxesImage`       ax.images
+# `~.axes.Axes.legend` - Axes legends        `.Legend`          ax.legends
+# `~.axes.Axes.plot` - xy plots              `.Line2D`          ax.lines
+# `~.axes.Axes.scatter` - scatter charts     `.PolyCollection`  ax.collections
+# `~.axes.Axes.text` - text                  `.Text`            ax.texts
+# =========================================  =================  ===============
 #
 #
 # In addition to all of these ``Artists``, the ``Axes`` contains two
@@ -510,20 +571,20 @@ plt.show()
 #
 # Below is a summary of the Artists that the Axes contains
 #
-# ==============    ======================================
+# ==============    =========================================
 # Axes attribute    Description
-# ==============    ======================================
-# artists           A list of Artist instances
-# patch             Rectangle instance for Axes background
-# collections       A list of Collection instances
-# images            A list of AxesImage
-# legends           A list of Legend instances
-# lines             A list of Line2D instances
-# patches           A list of Patch instances
-# texts             A list of Text instances
-# xaxis             matplotlib.axis.XAxis instance
-# yaxis             matplotlib.axis.YAxis instance
-# ==============    ======================================
+# ==============    =========================================
+# artists           A list of `.Artist` instances
+# patch             `.Rectangle` instance for Axes background
+# collections       A list of `.Collection` instances
+# images            A list of `.AxesImage`
+# legends           A list of `.Legend` instances
+# lines             A list of `.Line2D` instances
+# patches           A list of `.Patch` instances
+# texts             A list of `.Text` instances
+# xaxis             A `matplotlib.axis.XAxis` instance
+# yaxis             A `matplotlib.axis.YAxis` instance
+# ==============    =========================================
 #
 # .. _axis-container:
 #
@@ -578,28 +639,35 @@ axis.get_ticklines(minor=True)
 # (these have corresponding setters where useful, such as
 # :meth:`~matplotlib.axis.Axis.set_major_formatter`.)
 #
-# ====================  =====================================================
-# Accessor method       Description
-# ====================  =====================================================
-# get_scale             The scale of the axis, e.g., 'log' or 'linear'
-# get_view_interval     The interval instance of the axis view limits
-# get_data_interval     The interval instance of the axis data limits
-# get_gridlines         A list of grid lines for the Axis
-# get_label             The axis label - a Text instance
-# get_ticklabels        A list of Text instances - keyword minor=True|False
-# get_ticklines         A list of Line2D instances - keyword minor=True|False
-# get_ticklocs          A list of Tick locations - keyword minor=True|False
-# get_major_locator     The `.ticker.Locator` instance for major ticks
-# get_major_formatter   The `.ticker.Formatter` instance for major ticks
-# get_minor_locator     The `.ticker.Locator` instance for minor ticks
-# get_minor_formatter   The `.ticker.Formatter` instance for minor ticks
-# get_major_ticks       A list of Tick instances for major ticks
-# get_minor_ticks       A list of Tick instances for minor ticks
-# grid                  Turn the grid on or off for the major or minor ticks
-# ====================  =====================================================
+# =============================  ==============================================
+# Axis accessor method           Description
+# =============================  ==============================================
+# `~.Axis.get_scale`             The scale of the Axis, e.g., 'log' or 'linear'
+# `~.Axis.get_view_interval`     The interval instance of the Axis view limits
+# `~.Axis.get_data_interval`     The interval instance of the Axis data limits
+# `~.Axis.get_gridlines`         A list of grid lines for the Axis
+# `~.Axis.get_label`             The Axis label - a `.Text` instance
+# `~.Axis.get_offset_text`       The Axis offset text - a `.Text` instance
+# `~.Axis.get_ticklabels`        A list of `.Text` instances -
+#                                keyword minor=True|False
+# `~.Axis.get_ticklines`         A list of `.Line2D` instances -
+#                                keyword minor=True|False
+# `~.Axis.get_ticklocs`          A list of Tick locations -
+#                                keyword minor=True|False
+# `~.Axis.get_major_locator`     The `.ticker.Locator` instance for major ticks
+# `~.Axis.get_major_formatter`   The `.ticker.Formatter` instance for major
+#                                ticks
+# `~.Axis.get_minor_locator`     The `.ticker.Locator` instance for minor ticks
+# `~.Axis.get_minor_formatter`   The `.ticker.Formatter` instance for minor
+#                                ticks
+# `~.axis.Axis.get_major_ticks`  A list of `.Tick` instances for major ticks
+# `~.axis.Axis.get_minor_ticks`  A list of `.Tick` instances for minor ticks
+# `~.Axis.grid`                  Turn the grid on or off for the major or minor
+#                                ticks
+# =============================  ==============================================
 #
 # Here is an example, not recommended for its beauty, which customizes
-# the axes and tick properties
+# the Axes and Tick properties.
 
 # plt.figure creates a matplotlib.figure.Figure instance
 fig = plt.figure()
@@ -639,15 +707,15 @@ plt.show()
 # and lower ticks.  Each of these is accessible directly as an attribute
 # of the ``Tick``.
 #
-# ==============   ==========================================================
-# Tick attribute   Description
-# ==============   ==========================================================
-# tick1line        Line2D instance
-# tick2line        Line2D instance
-# gridline         Line2D instance
-# label1           Text instance
-# label2           Text instance
-# ==============   ==========================================================
+# ==============  ==========================================================
+# Tick attribute  Description
+# ==============  ==========================================================
+# tick1line       A `.Line2D` instance
+# tick2line       A `.Line2D` instance
+# gridline        A `.Line2D` instance
+# label1          A `.Text` instance
+# label2          A `.Text` instance
+# ==============  ==========================================================
 #
 # Here is an example which sets the formatter for the right side ticks with
 # dollar signs and colors them green on the right side of the yaxis.
